=== added file 'README'
--- README	1970-01-01 00:00:00 +0000
+++ README	2005-03-09 04:08:15 +0000
@@ -0,0 +1,26 @@
+Release notes for Bazaar-NG (pre-0)
+
+mbp@sourcefrog.net, November 2004, London
+
+
+* There is little locking or transaction control here; if you
+  interrupt it the tree may be arbitrarily broken.  This will be
+  fixed.
+
+* Don't use this for critical data; at the very least keep separate
+  regular snapshots of your tree.
+
+
+Dependencies
+------------
+
+This is mostly developed on Linux (Ubuntu); it should work on Unix,
+Windows, or OS X with relatively little trouble.
+
+bzr requires a fairly recent Python, say after 2.2.  
+
+You can improve performance by installing Fredrik Lundh's amazing
+cElementTree__ module.
+
+__ http://effbot.org/zone/celementtree.htm
+

=== added file 'bzr.py'
--- bzr.py	1970-01-01 00:00:00 +0000
+++ bzr.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,737 @@
+#! /usr/bin/python
+
+
+# Copyright (C) 2004, 2005 by Martin Pool
+# Copyright (C) 2005 by Canonical Ltd
+
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Bazaar-NG -- a free distributed version-control tool
+
+**WARNING: THIS IS AN UNSTABLE DEVELOPMENT VERSION**
+
+Current limitation include:
+
+* Metadata format is not stable yet -- you may need to
+  discard history in the future.
+
+* No handling of subdirectories, symlinks or any non-text files.
+
+* Insufficient error handling.
+
+* Many commands unimplemented or partially implemented.
+
+* Space-inefficient storage.
+
+* No merge operators yet.
+
+Interesting commands::
+
+  bzr help
+       Show summary help screen
+  bzr version
+       Show software version/licence/non-warranty.
+  bzr init
+       Start versioning the current directory
+  bzr add FILE...
+       Make files versioned.
+  bzr log
+       Show revision history.
+  bzr diff
+       Show changes from last revision to working copy.
+  bzr commit -m 'MESSAGE'
+       Store current state as new revision.
+  bzr export REVNO DESTINATION
+       Export the branch state at a previous version.
+  bzr status
+       Show summary of pending changes.
+  bzr remove FILE...
+       Make a file not versioned.
+"""
+
+# not currently working:
+#  bzr check
+#       Run internal consistency checks.
+#  bzr info
+#       Show some information about this branch.
+
+
+
+__copyright__ = "Copyright 2005 Canonical Development Ltd."
+__author__ = "Martin Pool <mbp@canonical.com>"
+__docformat__ = "restructuredtext en"
+__version__ = '0.0.0'
+
+
+import sys, os, random, time, sha, sets, types, re, shutil, tempfile
+import traceback, socket, fnmatch, difflib
+from os import path
+from sets import Set
+from pprint import pprint
+from stat import *
+from glob import glob
+from ElementTree import Element, ElementTree, SubElement
+
+import bzrlib
+from bzrlib.store import ImmutableStore
+from bzrlib.trace import mutter, note, log_error
+from bzrlib.errors import bailout, BzrError
+from bzrlib.osutils import quotefn, pumpfile, isdir, isfile
+from bzrlib.tree import RevisionTree, EmptyTree, WorkingTree, Tree
+from bzrlib.revision import Revision
+from bzrlib import Branch, Inventory, InventoryEntry, ScratchBranch, BZRDIR, \
+     format_date
+
+BZR_DIFF_FORMAT = "## Bazaar-NG diff, format 0 ##\n"
+BZR_PATCHNAME_FORMAT = 'cset:sha1:%s'
+
+## standard representation
+NONE_STRING = '(none)'
+EMPTY = 'empty'
+
+
+## TODO: Perhaps a different version of inventory commands that
+## returns iterators...
+
+## TODO: Perhaps an AtomicFile class that writes to a temporary file and then renames.
+
+## TODO: Some kind of locking on branches.  Perhaps there should be a
+## parameter to the branch object saying whether we want a read or
+## write lock; release it from destructor.  Perhaps don't even need a
+## read lock to look at immutable objects?
+
+## TODO: Perhaps make UUIDs predictable in test mode to make it easier
+## to compare output?
+
+## TODO: Is ElementTree really all that much better for our purposes?
+## Perhaps using the standard MiniDOM would be enough?
+
+
+
+
+
+
+######################################################################
+# check status
+
+
+def cmd_status(all=False):
+    """Display status summary.
+
+    For each file there is a single line giving its file state and name.
+    The name is that in the current revision unless it is deleted or
+    missing, in which case the old name is shown.
+
+    :todo: Don't show unchanged files unless ``--all`` is given?
+    """
+    Branch('.').show_status(show_all=all)
+
+
+
+######################################################################
+# examining history
+def cmd_get_revision(revision_id):
+    Branch('.').get_revision(revision_id).write_xml(sys.stdout)
+
+
+def cmd_get_inventory(inventory_id):
+    """Return inventory in XML by hash"""
+    Branch('.').get_inventory(inventory_hash).write_xml(sys.stdout)
+
+
+def cmd_get_revision_inventory(revision_id):
+    """Output inventory for a revision."""
+    b = Branch('.')
+    b.get_revision_inventory(revision_id).write_xml(sys.stdout)
+
+
+def cmd_get_file_text(text_id):
+    """Get contents of a file by hash."""
+    sf = Branch('.').text_store[text_id]
+    pumpfile(sf, sys.stdout)
+
+
+
+######################################################################
+# commands
+    
+
+def cmd_revno():
+    """Show number of revisions on this branch"""
+    print Branch('.').revno()
+    
+
+def cmd_add(file_list, verbose=False):
+    """Add specified files.
+    
+    Fails if the files are already added.
+    """
+    Branch('.').add(file_list, verbose=verbose)
+
+
+def cmd_inventory(revision=None):
+    """Show inventory of the current working copy."""
+    ## TODO: Also optionally show a previous inventory
+    ## TODO: Format options
+    b = Branch('.')
+    if revision == None:
+        inv = b.read_working_inventory()
+    else:
+        inv = b.get_revision_inventory(b.lookup_revision(revision))
+        
+    for path, entry in inv.iter_entries():
+        print '%-50s %s' % (entry.file_id, path)
+
+
+
+def cmd_info():
+    b = Branch('.')
+    print 'branch format:', b.controlfile('branch-format', 'r').readline().rstrip('\n')
+    print 'revision number:', b.revno()
+    print 'number of versioned files:', len(b.read_working_inventory())
+
+
+def cmd_remove(file_list, verbose=False):
+    Branch('.').remove(file_list, verbose=verbose)
+
+
+
+def cmd_file_id(filename):
+    i = Branch('.').read_working_inventory().path2id(filename)
+    if i is None:
+        bailout("%s is not a versioned file" % filename)
+    else:
+        print i
+
+
+def cmd_find_filename(fileid):
+    n = find_filename(fileid)
+    if n is None:
+        bailout("%s is not a live file id" % fileid)
+    else:
+        print n
+
+
+def cmd_revision_history():
+    for patchid in Branch('.').revision_history():
+        print patchid
+
+
+
+def cmd_init():
+    # TODO: Check we're not already in a working directory?  At the
+    # moment you'll get an ugly error.
+    
+    # TODO: What if we're in a subdirectory of a branch?  Would like
+    # to allow that, but then the parent may need to understand that
+    # the children have disappeared, or should they be versioned in
+    # both?
+
+    # TODO: Take an argument/option for branch name.
+    Branch('.', init=True)
+
+
+def cmd_diff(revision=None):
+    """Show diff from basis to working copy.
+
+    :todo: Take one or two revision arguments, look up those trees,
+           and diff them.
+
+    :todo: Allow diff across branches.
+
+    :todo: Mangle filenames in diff to be more relevant.
+
+    :todo: Shouldn't be in the cmd function.
+    """
+
+    b = Branch('.')
+
+    if revision == None:
+        old_tree = b.basis_tree()
+    else:
+        old_tree = b.revision_tree(b.lookup_revision(revision))
+        
+    new_tree = b.working_tree()
+    old_inv = old_tree.inventory
+    new_inv = new_tree.inventory
+
+    # TODO: Options to control putting on a prefix or suffix, perhaps as a format string
+    old_label = ''
+    new_label = ''
+
+    DEVNULL = '/dev/null'
+    # Windows users, don't panic about this filename -- it is a
+    # special signal to GNU patch that the file should be created or
+    # deleted respectively.
+
+    # TODO: Generation of pseudo-diffs for added/deleted files could
+    # be usefully made into a much faster special case.
+
+    # TODO: Better to return them in sorted order I think.
+    
+    for file_state, fid, old_name, new_name, kind in bzrlib.diff_trees(old_tree, new_tree):
+        d = None
+
+        # Don't show this by default; maybe do it if an option is passed
+        # idlabel = '      {%s}' % fid
+        idlabel = ''
+
+        # FIXME: Something about the diff format makes patch unhappy
+        # with newly-added files.
+
+        def diffit(*a, **kw):
+            sys.stdout.writelines(difflib.unified_diff(*a, **kw))
+            print
+        
+        if file_state in ['.', '?', 'I']:
+            continue
+        elif file_state == 'A':
+            print '*** added %s %r' % (kind, new_name)
+            if kind == 'file':
+                diffit([],
+                       new_tree.get_file(fid).readlines(),
+                       fromfile=DEVNULL,
+                       tofile=new_label + new_name + idlabel)
+        elif file_state == 'D':
+            assert isinstance(old_name, types.StringTypes)
+            print '*** deleted %s %r' % (kind, old_name)
+            if kind == 'file':
+                diffit(old_tree.get_file(fid).readlines(), [],
+                       fromfile=old_label + old_name + idlabel,
+                       tofile=DEVNULL)
+        elif file_state in ['M', 'R']:
+            if file_state == 'M':
+                assert kind == 'file'
+                assert old_name == new_name
+                print '*** modified %s %r' % (kind, new_name)
+            elif file_state == 'R':
+                print '*** renamed %s %r => %r' % (kind, old_name, new_name)
+
+            if kind == 'file':
+                diffit(old_tree.get_file(fid).readlines(),
+                       new_tree.get_file(fid).readlines(),
+                       fromfile=old_label + old_name + idlabel,
+                       tofile=new_label + new_name)
+        else:
+            bailout("can't represent state %s {%s}" % (file_state, fid))
+
+
+
+def cmd_log():
+    """Show log of this branch.
+
+    :todo: Options for utc; to show ids; to limit range; etc.
+    """
+    Branch('.').write_log()
+
+
+def cmd_ls(revision=None, verbose=False):
+    """List files in a tree.
+
+    :todo: Take a revision or remote path and list that tree instead.
+    """
+    b = Branch('.')
+    if revision == None:
+        tree = b.working_tree()
+    else:
+        tree = b.revision_tree(b.lookup_revision(revision))
+        
+    for fp, fc, kind, fid in tree.list_files():
+        if verbose:
+            if kind == 'directory':
+                kindch = '/'
+            elif kind == 'file':
+                kindch = ''
+            else:
+                kindch = '???'
+                
+            print '%-8s %s%s' % (fc, fp, kindch)
+        else:
+            print fp
+    
+    
+
+def cmd_unknowns():
+    """List unknown files"""
+    for f in Branch('.').unknowns():
+        print quotefn(f)
+
+
+def cmd_lookup_revision(revno):
+    try:
+        revno = int(revno)
+    except ValueError:
+        bailout("usage: lookup-revision REVNO",
+                ["REVNO is a non-negative revision number for this branch"])
+
+    print Branch('.').lookup_revision(revno) or NONE_STRING
+
+
+
+def cmd_export(revno, dest):
+    """Export past revision to destination directory."""
+    b = Branch('.')
+    rh = b.lookup_revision(int(revno))
+    t = b.revision_tree(rh)
+    t.export(dest)
+
+
+
+######################################################################
+# internal/test commands
+
+
+def cmd_uuid():
+    """Print a newly-generated UUID."""
+    print uuid()
+
+
+
+def cmd_commit(message, verbose=False):
+    Branch('.').commit(message, verbose=verbose)
+
+
+def cmd_check():
+    """Check consistency of the branch."""
+    check()
+
+
+def cmd_is(pred, *rest):
+    """Test whether PREDICATE is true."""
+    try:
+        cmd_handler = globals()['assert_' + pred.replace('-', '_')]
+    except KeyError:
+        bailout("unknown predicate: %s" % quotefn(pred))
+        
+    try:
+        cmd_handler(*rest)
+    except BzrCheckError:
+        # by default we don't print the message so that this can
+        # be used from shell scripts without producing noise
+        sys.exit(1)
+
+
+def cmd_username():
+    print bzrlib.osutils.username()
+
+
+def cmd_user_email():
+    print bzrlib.osutils.user_email()
+
+
+def cmd_gen_revision_id():
+    import time
+    print bzrlib.branch._gen_revision_id(time.time())
+
+
+def cmd_doctest():
+    """Run internal doctest suite"""
+    ## -v, if present, is seen by doctest; the argument is just here
+    ## so our parser doesn't complain
+
+    ## TODO: --verbose option
+    
+    import bzr, doctest, bzrlib.store
+    bzrlib.trace.verbose = False
+    doctest.testmod(bzr)
+    doctest.testmod(bzrlib.store)
+    doctest.testmod(bzrlib.inventory)
+    doctest.testmod(bzrlib.branch)
+    doctest.testmod(bzrlib.osutils)
+    doctest.testmod(bzrlib.tree)
+
+    # more strenuous tests;
+    import bzrlib.tests
+    doctest.testmod(bzrlib.tests)
+
+
+######################################################################
+# help
+
+
+def cmd_help():
+    # TODO: Specific help for particular commands
+    print __doc__
+
+
+def cmd_version():
+    print "bzr (bazaar-ng) %s" % __version__
+    print __copyright__
+    print "http://bazaar-ng.org/"
+    print
+    print \
+"""bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and
+you may use, modify and redistribute it under the terms of the GNU 
+General Public License version 2 or later."""
+
+
+def cmd_rocks():
+    """Statement of optimism."""
+    print "it sure does!"
+
+
+
+######################################################################
+# main routine
+
+
+# list of all available options; the rhs can be either None for an
+# option that takes no argument, or a constructor function that checks
+# the type.
+OPTIONS = {
+    'all':                    None,
+    'help':                   None,
+    'message':                unicode,
+    'revision':               int,
+    'show-ids':               None,
+    'verbose':                None,
+    'version':                None,
+    }
+
+SHORT_OPTIONS = {
+    'm':                      'message',
+    'r':                      'revision',
+    'v':                      'verbose',
+}
+
+# List of options that apply to particular commands; commands not
+# listed take none.
+cmd_options = {
+    'add':                    ['verbose'],
+    'commit':                 ['message', 'verbose'],
+    'diff':                   ['revision'],
+    'inventory':              ['revision'],
+    'ls':                     ['revision', 'verbose'],
+    'status':                 ['all'],
+    'log':                    ['show-ids'],
+    'remove':                 ['verbose'],
+    }
+
+
+cmd_args = {
+    'init':                   [],
+    'add':                    ['file+'],
+    'commit':                 [],
+    'diff':                   [],
+    'file-id':                ['filename'],
+    'get-file-text':          ['text_id'],
+    'get-inventory':          ['inventory_id'],
+    'get-revision':           ['revision_id'],
+    'get-revision-inventory': ['revision_id'],
+    'log':                    [],
+    'lookup-revision':        ['revno'],
+    'export':                 ['revno', 'dest'],
+    'remove':                 ['file+'],
+    'status':                 [],
+    }
+
+
+def parse_args(argv):
+    """Parse command line.
+    
+    Arguments and options are parsed at this level before being passed
+    down to specific command handlers.  This routine knows, from a
+    lookup table, something about the available options, what optargs
+    they take, and which commands will accept them.
+
+    >>> parse_args('bzr --help'.split())
+    ([], {'help': True})
+    >>> parse_args('bzr --version'.split())
+    ([], {'version': True})
+    >>> parse_args('bzr status --all'.split())
+    (['status'], {'all': True})
+    """
+    args = []
+    opts = {}
+
+    # TODO: Maybe handle '--' to end options?
+
+    it = iter(argv[1:])
+    while it:
+        a = it.next()
+        if a[0] == '-':
+            if a[1] == '-':
+                mutter("  got option %r" % a)
+                optname = a[2:]
+                if optname not in OPTIONS:
+                    bailout('unknown long option %r' % a)
+            else:
+                shortopt = a[1:]
+                if shortopt not in SHORT_OPTIONS:
+                    bailout('unknown short option %r' % a)
+                optname = SHORT_OPTIONS[shortopt]
+            
+            if optname in opts:
+                # XXX: Do we ever want to support this, e.g. for -r?
+                bailout('repeated option %r' % a)
+            optargfn = OPTIONS[optname]
+            if optargfn:
+                if not it:
+                    bailout('option %r needs an argument' % a)
+                opts[optname] = optargfn(it.next())
+                mutter("    option argument %r" % opts[optname])
+            else:
+                # takes no option argument
+                opts[optname] = True
+        elif a[:1] == '-':
+            bailout('unknown short option %r' % a)
+        else:
+            args.append(a)
+
+    return args, opts
+
+
+
+def _match_args(cmd, args):
+    """Check non-option arguments match required pattern.
+
+    >>> _match_args('status', ['asdasdsadasd'])
+    Traceback (most recent call last):
+    ...
+    BzrError: ("extra arguments to command status: ['asdasdsadasd']", [])
+    >>> _match_args('add', ['asdasdsadasd'])
+    {'file_list': ['asdasdsadasd']}
+    >>> _match_args('add', 'abc def gj'.split())
+    {'file_list': ['abc', 'def', 'gj']}
+    """
+    # match argument pattern
+    argform = cmd_args.get(cmd, [])
+    argdict = {}
+    # TODO: Need a way to express 'cp SRC... DEST', where it matches
+    # all but one.
+    for ap in argform:
+        argname = ap[:-1]
+        if ap[-1] == '?':
+            assert 0
+        elif ap[-1] == '*':
+            assert 0
+        elif ap[-1] == '+':
+            if not args:
+                bailout("command %r needs one or more %s"
+                        % (cmd, argname.upper()))
+            else:
+                argdict[argname + '_list'] = args[:]
+                args = []
+        else:
+            # just a plain arg
+            argname = ap
+            if not args:
+                bailout("command %r requires argument %s"
+                        % (cmd, argname.upper()))
+            else:
+                argdict[argname] = args.pop(0)
+            
+    if args:
+        bailout("extra arguments to command %s: %r"
+                % (cmd, args))
+
+    return argdict
+
+
+
+def run_bzr(argv):
+    """Execute a command.
+
+    This is similar to main(), but without all the trappings for
+    logging and error handling.
+    """
+    try:
+        args, opts = parse_args(argv)
+        if 'help' in opts:
+            # TODO: pass down other arguments in case they asked for
+            # help on a command name?
+            cmd_help()
+            return 0
+        elif 'version' in opts:
+            cmd_version()
+            return 0
+        cmd = args.pop(0)
+    except IndexError:
+        log_error('usage: bzr COMMAND\n')
+        log_error('  try "bzr help"\n')
+        return 1
+            
+    try:
+        cmd_handler = globals()['cmd_' + cmd.replace('-', '_')]
+    except KeyError:
+        bailout("unknown command " + `cmd`)
+
+    # TODO: special --profile option to turn on the Python profiler
+
+    # check options are reasonable
+    allowed = cmd_options.get(cmd, [])
+    for oname in opts:
+        if oname not in allowed:
+            bailout("option %r is not allowed for command %r"
+                    % (oname, cmd))
+
+    cmdargs = _match_args(cmd, args)
+    cmdargs.update(opts)
+
+    ret = cmd_handler(**cmdargs) or 0
+
+
+
+def main(argv):
+    ## TODO: Handle command-line options; probably know what options are valid for
+    ## each command
+
+    ## TODO: If the arguments are wrong, give a usage message rather
+    ## than just a backtrace.
+
+    try:
+        t = bzrlib.trace._tracefile
+        t.write('-' * 60 + '\n')
+        t.write('bzr invoked at %s\n' % format_date(time.time()))
+        t.write('  by %s on %s\n' % (bzrlib.osutils.username(), socket.gethostname()))
+        t.write('  arguments: %r\n' % argv)
+
+        starttime = os.times()[4]
+
+        import platform
+        t.write('  platform: %s\n' % platform.platform())
+        t.write('  python: %s\n' % platform.python_version())
+
+        ret = run_bzr(argv)
+        
+        times = os.times()
+        mutter("finished, %.3fu/%.3fs cpu, %.3fu/%.3fs cum"
+               % times[:4])
+        mutter("    %.3f elapsed" % (times[4] - starttime))
+
+        return ret
+    except BzrError, e:
+        log_error('bzr: error: ' + e.args[0] + '\n')
+        if len(e.args) > 1:
+            for h in e.args[1]:
+                log_error('  ' + h + '\n')
+        return 1
+    except Exception, e:
+        log_error('bzr: exception: %s\n' % e)
+        log_error('    see .bzr.log for details\n')
+        traceback.print_exc(None, bzrlib.trace._tracefile)
+        traceback.print_exc(None, sys.stderr)
+        return 1
+
+    # TODO: Maybe nicer handling of IOError?
+
+
+
+if __name__ == '__main__':
+    sys.exit(main(sys.argv))
+    ##import profile
+    ##profile.run('main(sys.argv)')
+    

=== added directory 'bzrlib'
=== added file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/__init__.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,29 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""bzr library"""
+
+from inventory import Inventory, InventoryEntry
+from branch import Branch, ScratchBranch
+from osutils import format_date
+from tree import Tree
+from diff import diff_trees
+
+BZRDIR = ".bzr"
+
+DEFAULT_IGNORE = ['.*', '*~', '#*#', '*.tmp', '*.o', '*.a']
+

=== added file 'bzrlib/branch.py'
--- bzrlib/branch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/branch.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,761 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from sets import Set
+
+import sys, os, os.path, random, time, sha, sets, types, re, shutil, tempfile
+import traceback, socket, fnmatch, difflib, time
+from binascii import hexlify
+
+import bzrlib
+from inventory import Inventory
+from trace import mutter, note
+from tree import Tree, EmptyTree, RevisionTree, WorkingTree
+from inventory import InventoryEntry, Inventory
+from osutils import isdir, quotefn, isfile, uuid, sha_file, username, chomp, \
+     format_date, compact_date, pumpfile, user_email, rand_bytes, splitpath, \
+     joinpath, sha_string, file_kind
+from store import ImmutableStore
+from revision import Revision
+from errors import bailout
+from textui import show_status
+from diff import diff_trees
+
+BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
+## TODO: Maybe include checks for common corruption of newlines, etc?
+
+
+
+
+
+######################################################################
+# branch objects
+
+class Branch:
+    """Branch holding a history of revisions.
+
+    :todo: Perhaps use different stores for different classes of object,
+           so that we can keep track of how much space each one uses,
+           or garbage-collect them.
+
+    :todo: Add a RemoteBranch subclass.  For the basic case of read-only
+           HTTP access this should be very easy by, 
+           just redirecting controlfile access into HTTP requests.
+           We would need a RemoteStore working similarly.
+
+    :todo: Keep the on-disk branch locked while the object exists.
+
+    :todo: mkdir() method.
+    """
+    def __init__(self, base, init=False):
+        """Create new branch object at a particular location.
+
+        :param base: Base directory for the branch.
+
+        :param init: If True, create new control files in a previously
+             unversioned directory.  If False, the branch must already
+             be versioned.
+
+        In the test suite, creation of new trees is tested using the
+        `ScratchBranch` class.
+        """
+        self.base = os.path.realpath(base)
+        if init:
+            self._make_control()
+        else:
+            if not isdir(self.controlfilename('.')):
+                bailout("not a bzr branch: %s" % quotefn(base),
+                        ['use "bzr init" to initialize a new working tree',
+                         'current bzr can only operate from top-of-tree'])
+            self._check_format()
+
+        self.text_store = ImmutableStore(self.controlfilename('text-store'))
+        self.revision_store = ImmutableStore(self.controlfilename('revision-store'))
+        self.inventory_store = ImmutableStore(self.controlfilename('inventory-store'))
+
+
+    def __str__(self):
+        return '%s(%r)' % (self.__class__.__name__, self.base)
+
+
+    __repr__ = __str__
+
+
+    def _rel(self, name):
+        """Return filename relative to branch top"""
+        return os.path.join(self.base, name)
+        
+
+    def controlfilename(self, file_or_path):
+        """Return location relative to branch."""
+        if isinstance(file_or_path, types.StringTypes):
+            file_or_path = [file_or_path]
+        return os.path.join(self.base, bzrlib.BZRDIR, *file_or_path)
+
+
+    def controlfile(self, file_or_path, mode='r'):
+        """Open a control file for this branch"""
+        return file(self.controlfilename(file_or_path), mode)
+
+
+    def _make_control(self):
+        os.mkdir(self.controlfilename([]))
+        self.controlfile('README', 'w').write(
+            "This is a Bazaar-NG control directory.\n"
+            "Do not change any files in this directory.")
+        self.controlfile('branch-format', 'w').write(BZR_BRANCH_FORMAT)
+        for d in ('text-store', 'inventory-store', 'revision-store'):
+            os.mkdir(self.controlfilename(d))
+        for f in ('revision-history', 'merged-patches',
+                  'pending-merged-patches', 'branch-name'):
+            self.controlfile(f, 'w').write('')
+        mutter('created control directory in ' + self.base)
+        Inventory().write_xml(self.controlfile('inventory','w'))
+
+
+    def _check_format(self):
+        """Check this branch format is supported.
+
+        The current tool only supports the current unstable format.
+
+        In the future, we might need different in-memory Branch
+        classes to support downlevel branches.  But not yet.
+        """        
+        # read in binary mode to detect newline wierdness.
+        fmt = self.controlfile('branch-format', 'rb').read()
+        if fmt != BZR_BRANCH_FORMAT:
+            bailout('sorry, branch format %r not supported' % fmt,
+                    ['use a different bzr version',
+                     'or remove the .bzr directory and "bzr init" again'])
+
+
+    def read_working_inventory(self):
+        """Read the working inventory."""
+        before = time.time()
+        inv = Inventory.read_xml(self.controlfile('inventory', 'r'))
+        mutter("loaded inventory of %d items in %f"
+               % (len(inv), time.time() - before))
+        return inv
+
+
+    def _write_inventory(self, inv):
+        """Update the working inventory.
+
+        That is to say, the inventory describing changes underway, that
+        will be committed to the next revision.
+        """
+        inv.write_xml(self.controlfile('inventory', 'w'))
+        mutter('wrote inventory to %s' % quotefn(self.controlfilename('inventory')))
+
+
+    inventory = property(read_working_inventory, _write_inventory, None,
+                         """Inventory for the working copy.""")
+
+
+    def add(self, files, verbose=False):
+        """Make files versioned.
+
+        This puts the files in the Added state, so that they will be
+        recorded by the next commit.
+
+        :todo: Perhaps have an option to add the ids even if the files do
+               not (yet) exist.
+
+        :todo: Perhaps return the ids of the files?  But then again it
+               is easy to retrieve them if they're needed.
+
+        :todo: Option to specify file id.
+
+        :todo: Adding a directory should optionally recurse down and
+               add all non-ignored children.  Perhaps do that in a
+               higher-level method.
+
+        >>> b = ScratchBranch(files=['foo'])
+        >>> 'foo' in b.unknowns()
+        True
+        >>> b.show_status()
+        ?       foo
+        >>> b.add('foo')
+        >>> 'foo' in b.unknowns()
+        False
+        >>> bool(b.inventory.path2id('foo'))
+        True
+        >>> b.show_status()
+        A       foo
+
+        >>> b.add('foo')
+        Traceback (most recent call last):
+        ...
+        BzrError: ('foo is already versioned', [])
+
+        >>> b.add(['nothere'])
+        Traceback (most recent call last):
+        BzrError: ('cannot add: not a regular file or directory: nothere', [])
+        """
+
+        # TODO: Re-adding a file that is removed in the working copy
+        # should probably put it back with the previous ID.
+        if isinstance(files, types.StringTypes):
+            files = [files]
+        
+        inv = self.read_working_inventory()
+        for f in files:
+            if is_control_file(f):
+                bailout("cannot add control file %s" % quotefn(f))
+
+            fp = splitpath(f)
+
+            if len(fp) == 0:
+                bailout("cannot add top-level %r" % f)
+                
+            fullpath = os.path.normpath(self._rel(f))
+
+            if isfile(fullpath):
+                kind = 'file'
+            elif isdir(fullpath):
+                kind = 'directory'
+            else:
+                bailout('cannot add: not a regular file or directory: %s' % quotefn(f))
+
+            if len(fp) > 1:
+                parent_name = joinpath(fp[:-1])
+                mutter("lookup parent %r" % parent_name)
+                parent_id = inv.path2id(parent_name)
+                if parent_id == None:
+                    bailout("cannot add: parent %r is not versioned"
+                            % joinpath(fp[:-1]))
+            else:
+                parent_id = None
+
+            file_id = _gen_file_id(fp[-1])
+            inv.add(InventoryEntry(file_id, fp[-1], kind=kind, parent_id=parent_id))
+            if verbose:
+                show_status('A', kind, quotefn(f))
+                
+            mutter("add file %s file_id:{%s} kind=%r parent_id={%s}"
+                   % (f, file_id, kind, parent_id))
+        self._write_inventory(inv)
+
+
+
+    def remove(self, files, verbose=False):
+        """Mark nominated files for removal from the inventory.
+
+        This does not remove their text.  This does not run on 
+
+        :todo: Refuse to remove modified files unless --force is given?
+
+        >>> b = ScratchBranch(files=['foo'])
+        >>> b.add('foo')
+        >>> b.inventory.has_filename('foo')
+        True
+        >>> b.remove('foo')
+        >>> b.working_tree().has_filename('foo')
+        True
+        >>> b.inventory.has_filename('foo')
+        False
+        
+        >>> b = ScratchBranch(files=['foo'])
+        >>> b.add('foo')
+        >>> b.commit('one')
+        >>> b.remove('foo')
+        >>> b.commit('two')
+        >>> b.inventory.has_filename('foo') 
+        False
+        >>> b.basis_tree().has_filename('foo') 
+        False
+        >>> b.working_tree().has_filename('foo') 
+        True
+
+        :todo: Do something useful with directories.
+
+        :todo: Should this remove the text or not?  Tough call; not
+        removing may be useful and the user can just use use rm, and
+        is the opposite of add.  Removing it is consistent with most
+        other tools.  Maybe an option.
+        """
+        ## TODO: Normalize names
+        ## TODO: Remove nested loops; better scalability
+
+        if isinstance(files, types.StringTypes):
+            files = [files]
+        
+        inv = self.read_working_inventory()
+
+        # do this before any modifications
+        for f in files:
+            fid = inv.path2id(f)
+            if not fid:
+                bailout("cannot remove unversioned file %s" % quotefn(f))
+            mutter("remove inventory entry %s {%s}" % (quotefn(f), fid))
+            if verbose:
+                show_status('D', inv[fid].kind, quotefn(f))
+            del inv[fid]
+
+        self._write_inventory(inv)
+
+
+    def unknowns(self):
+        """Return all unknown files.
+
+        These are files in the working directory that are not versioned or
+        control files or ignored.
+        
+        >>> b = ScratchBranch(files=['foo', 'foo~'])
+        >>> list(b.unknowns())
+        ['foo']
+        >>> b.add('foo')
+        >>> list(b.unknowns())
+        []
+        >>> b.remove('foo')
+        >>> list(b.unknowns())
+        ['foo']
+        """
+        return self.working_tree().unknowns()
+
+
+    def commit(self, message, timestamp=None, committer=None,
+               verbose=False):
+        """Commit working copy as a new revision.
+        
+        The basic approach is to add all the file texts into the
+        store, then the inventory, then make a new revision pointing
+        to that inventory and store that.
+        
+        This is not quite safe if the working copy changes during the
+        commit; for the moment that is simply not allowed.  A better
+        approach is to make a temporary copy of the files before
+        computing their hashes, and then add those hashes in turn to
+        the inventory.  This should mean at least that there are no
+        broken hash pointers.  There is no way we can get a snapshot
+        of the whole directory at an instant.  This would also have to
+        be robust against files disappearing, moving, etc.  So the
+        whole thing is a bit hard.
+
+        :param timestamp: if not None, seconds-since-epoch for a
+             postdated/predated commit.
+        """
+
+        ## TODO: Show branch names
+
+        # TODO: Don't commit if there are no changes, unless forced?
+
+        # First walk over the working inventory; and both update that
+        # and also build a new revision inventory.  The revision
+        # inventory needs to hold the text-id, sha1 and size of the
+        # actual file versions committed in the revision.  (These are
+        # not present in the working inventory.)  We also need to
+        # detect missing/deleted files, and remove them from the
+        # working inventory.
+
+        work_inv = self.read_working_inventory()
+        inv = Inventory()
+        basis = self.basis_tree()
+        basis_inv = basis.inventory
+        missing_ids = []
+        for path, entry in work_inv.iter_entries():
+            ## TODO: Cope with files that have gone missing.
+
+            ## TODO: Check that the file kind has not changed from the previous
+            ## revision of this file (if any).
+
+            entry = entry.copy()
+
+            p = self._rel(path)
+            file_id = entry.file_id
+            mutter('commit prep file %s, id %r ' % (p, file_id))
+
+            if not os.path.exists(p):
+                mutter("    file is missing, removing from inventory")
+                if verbose:
+                    show_status('D', entry.kind, quotefn(path))
+                missing_ids.append(file_id)
+                continue
+
+            # TODO: Handle files that have been deleted
+
+            # TODO: Maybe a special case for empty files?  Seems a
+            # waste to store them many times.
+
+            inv.add(entry)
+
+            if basis_inv.has_id(file_id):
+                old_kind = basis_inv[file_id].kind
+                if old_kind != entry.kind:
+                    bailout("entry %r changed kind from %r to %r"
+                            % (file_id, old_kind, entry.kind))
+
+            if entry.kind == 'directory':
+                if not isdir(p):
+                    bailout("%s is entered as directory but not a directory" % quotefn(p))
+            elif entry.kind == 'file':
+                if not isfile(p):
+                    bailout("%s is entered as file but is not a file" % quotefn(p))
+
+                content = file(p, 'rb').read()
+
+                entry.text_sha1 = sha_string(content)
+                entry.text_size = len(content)
+
+                old_ie = basis_inv.has_id(file_id) and basis_inv[file_id]
+                if (old_ie
+                    and (old_ie.text_size == entry.text_size)
+                    and (old_ie.text_sha1 == entry.text_sha1)):
+                    ## assert content == basis.get_file(file_id).read()
+                    entry.text_id = basis_inv[file_id].text_id
+                    mutter('    unchanged from previous text_id {%s}' %
+                           entry.text_id)
+                    
+                else:
+                    entry.text_id = _gen_file_id(entry.name)
+                    self.text_store.add(content, entry.text_id)
+                    mutter('    stored with text_id {%s}' % entry.text_id)
+                    if verbose:
+                        if not old_ie:
+                            state = 'A'
+                        elif (old_ie.name == entry.name
+                              and old_ie.parent_id == entry.parent_id):
+                            state = 'R'
+                        else:
+                            state = 'M'
+
+                        show_status(state, entry.kind, quotefn(path))
+
+        for file_id in missing_ids:
+            # have to do this later so we don't mess up the iterator.
+            # since parents may be removed before their children we
+            # have to test.
+
+            # FIXME: There's probably a better way to do this; perhaps
+            # the workingtree should know how to filter itself.
+            if work_inv.has_id(file_id):
+                del work_inv[file_id]
+
+
+        inv_id = rev_id = _gen_revision_id(time.time())
+        
+        inv_tmp = tempfile.TemporaryFile()
+        inv.write_xml(inv_tmp)
+        inv_tmp.seek(0)
+        self.inventory_store.add(inv_tmp, inv_id)
+        mutter('new inventory_id is {%s}' % inv_id)
+
+        self._write_inventory(work_inv)
+
+        if timestamp == None:
+            timestamp = time.time()
+
+        if committer == None:
+            committer = username()
+
+        mutter("building commit log message")
+        rev = Revision(timestamp=timestamp,
+                       committer=committer,
+                       precursor = self.last_patch(),
+                       message = message,
+                       inventory_id=inv_id,
+                       revision_id=rev_id)
+
+        rev_tmp = tempfile.TemporaryFile()
+        rev.write_xml(rev_tmp)
+        rev_tmp.seek(0)
+        self.revision_store.add(rev_tmp, rev_id)
+        mutter("new revision_id is {%s}" % rev_id)
+        
+        ## XXX: Everything up to here can simply be orphaned if we abort
+        ## the commit; it will leave junk files behind but that doesn't
+        ## matter.
+
+        ## TODO: Read back the just-generated changeset, and make sure it
+        ## applies and recreates the right state.
+
+        ## TODO: Also calculate and store the inventory SHA1
+        mutter("committing patch r%d" % (self.revno() + 1))
+
+        mutter("append to revision-history")
+        self.controlfile('revision-history', 'at').write(rev_id + '\n')
+
+        mutter("done!")
+
+
+    def get_revision(self, revision_id):
+        """Return the Revision object for a named revision"""
+        r = Revision.read_xml(self.revision_store[revision_id])
+        assert r.revision_id == revision_id
+        return r
+
+
+    def get_inventory(self, inventory_id):
+        """Get Inventory object by hash.
+
+        :todo: Perhaps for this and similar methods, take a revision
+               parameter which can be either an integer revno or a
+               string hash."""
+        i = Inventory.read_xml(self.inventory_store[inventory_id])
+        return i
+
+
+    def get_revision_inventory(self, revision_id):
+        """Return inventory of a past revision."""
+        if revision_id == None:
+            return Inventory()
+        else:
+            return self.get_inventory(self.get_revision(revision_id).inventory_id)
+
+
+    def revision_history(self):
+        """Return sequence of revision hashes on to this branch.
+
+        >>> ScratchBranch().revision_history()
+        []
+        """
+        return [chomp(l) for l in self.controlfile('revision-history').readlines()]
+
+
+    def revno(self):
+        """Return current revision number for this branch.
+
+        That is equivalent to the number of revisions committed to
+        this branch.
+
+        >>> b = ScratchBranch()
+        >>> b.revno()
+        0
+        >>> b.commit('no foo')
+        >>> b.revno()
+        1
+        """
+        return len(self.revision_history())
+
+
+    def last_patch(self):
+        """Return last patch hash, or None if no history.
+
+        >>> ScratchBranch().last_patch() == None
+        True
+        """
+        ph = self.revision_history()
+        if ph:
+            return ph[-1]
+
+
+    def lookup_revision(self, revno):
+        """Return revision hash for revision number."""
+        if revno == 0:
+            return None
+
+        try:
+            # list is 0-based; revisions are 1-based
+            return self.revision_history()[revno-1]
+        except IndexError:
+            bailout("no such revision %s" % revno)
+
+
+    def revision_tree(self, revision_id):
+        """Return Tree for a revision on this branch.
+
+        `revision_id` may be None for the null revision, in which case
+        an `EmptyTree` is returned."""
+
+        if revision_id == None:
+            return EmptyTree()
+        else:
+            inv = self.get_revision_inventory(revision_id)
+            return RevisionTree(self.text_store, inv)
+
+
+    def working_tree(self):
+        """Return a `Tree` for the working copy."""
+        return WorkingTree(self.base, self.read_working_inventory())
+
+
+    def basis_tree(self):
+        """Return `Tree` object for last revision.
+
+        If there are no revisions yet, return an `EmptyTree`.
+
+        >>> b = ScratchBranch(files=['foo'])
+        >>> b.basis_tree().has_filename('foo')
+        False
+        >>> b.working_tree().has_filename('foo')
+        True
+        >>> b.add('foo')
+        >>> b.commit('add foo')
+        >>> b.basis_tree().has_filename('foo')
+        True
+        """
+        r = self.last_patch()
+        if r == None:
+            return EmptyTree()
+        else:
+            return RevisionTree(self.text_store, self.get_revision_inventory(r))
+
+
+
+    def write_log(self, utc=False):
+        """Write out human-readable log of commits to this branch
+
+        :param utc: If true, show dates in universal time, not local time."""
+        revno = 1
+        precursor = None
+        for p in self.revision_history():
+            print '-' * 40
+            print 'revno:', revno
+            ## TODO: Show hash if --id is given.
+            ##print 'revision-hash:', p
+            rev = self.get_revision(p)
+            print 'committer:', rev.committer
+            print 'timestamp: %s' % (format_date(rev.timestamp, utc))
+
+            ## opportunistic consistency check, same as check_patch_chaining
+            if rev.precursor != precursor:
+                bailout("mismatched precursor!")
+
+            print 'message:'
+            if not rev.message:
+                print '  (no message)'
+            else:
+                for l in rev.message.split('\n'):
+                    print '  ' + l
+
+            revno += 1
+            precursor = p
+
+
+
+    def show_status(branch, show_all=False):
+        """Display single-line status for non-ignored working files.
+
+        The list is show sorted in order by file name.
+
+        >>> b = ScratchBranch(files=['foo', 'foo~'])
+        >>> b.show_status()
+        ?       foo
+        >>> b.add('foo')
+        >>> b.show_status()
+        A       foo
+        >>> b.commit("add foo")
+        >>> b.show_status()
+
+        :todo: Get state for single files.
+
+        :todo: Perhaps show a slash at the end of directory names.        
+
+        """
+
+        # We have to build everything into a list first so that it can
+        # sorted by name, incorporating all the different sources.
+
+        # FIXME: Rather than getting things in random order and then sorting,
+        # just step through in order.
+
+        # Interesting case: the old ID for a file has been removed,
+        # but a new file has been created under that name.
+
+        old = branch.basis_tree()
+        old_inv = old.inventory
+        new = branch.working_tree()
+        new_inv = new.inventory
+
+        for fs, fid, oldname, newname, kind in diff_trees(old, new):
+            if fs == 'R':
+                show_status(fs, kind,
+                            oldname + ' => ' + newname)
+            elif fs == 'A' or fs == 'M':
+                show_status(fs, kind, newname)
+            elif fs == 'D':
+                show_status(fs, kind, oldname)
+            elif fs == '.':
+                if show_all:
+                    show_status(fs, kind, newname)
+            elif fs == 'I':
+                if show_all:
+                    show_status(fs, kind, newname)
+            elif fs == '?':
+                show_status(fs, kind, newname)
+            else:
+                bailout("wierd file state %r" % ((fs, fid),))
+                
+
+
+class ScratchBranch(Branch):
+    """Special test class: a branch that cleans up after itself.
+
+    >>> b = ScratchBranch()
+    >>> isdir(b.base)
+    True
+    >>> bd = b.base
+    >>> del b
+    >>> isdir(bd)
+    False
+    """
+    def __init__(self, files = []):
+        """Make a test branch.
+
+        This creates a temporary directory and runs init-tree in it.
+
+        If any files are listed, they are created in the working copy.
+        """
+        Branch.__init__(self, tempfile.mkdtemp(), init=True)
+        for f in files:
+            file(os.path.join(self.base, f), 'w').write('content of %s' % f)
+
+
+    def __del__(self):
+        """Destroy the test branch, removing the scratch directory."""
+        shutil.rmtree(self.base)
+
+    
+
+######################################################################
+# predicates
+
+
+def is_control_file(filename):
+    ## FIXME: better check
+    filename = os.path.normpath(filename)
+    while filename != '':
+        head, tail = os.path.split(filename)
+        ## mutter('check %r for control file' % ((head, tail), ))
+        if tail == bzrlib.BZRDIR:
+            return True
+        filename = head
+    return False
+
+
+
+def _gen_revision_id(when):
+    """Return new revision-id."""
+    s = '%s-%s-' % (user_email(), compact_date(when))
+    s += hexlify(rand_bytes(8))
+    return s
+
+
+def _gen_file_id(name):
+    """Return new file id.
+
+    This should probably generate proper UUIDs, but for the moment we
+    cope with just randomness because running uuidgen every time is
+    slow."""
+    assert '/' not in name
+    while name[0] == '.':
+        name = name[1:]
+    s = hexlify(rand_bytes(8))
+    return '-'.join((name, compact_date(time.time()), s))
+
+

=== added file 'bzrlib/check.py'
--- bzrlib/check.py	1970-01-01 00:00:00 +0000
+++ bzrlib/check.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,106 @@
+#! /usr/bin/python
+
+
+# Copyright (C) 2004, 2005 by Martin Pool
+# Copyright (C) 2005 by Canonical Ltd
+
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+######################################################################
+# consistency checks
+
+def check():
+    """Consistency check of tree."""
+    assert_in_tree()
+    mutter("checking tree")
+    check_patches_exist()
+    check_patch_chaining()
+    check_patch_uniqueness()
+    check_inventory()
+    mutter("tree looks OK")
+    ## TODO: Check that previous-inventory and previous-manifest
+    ## are the same as those stored in the previous changeset.
+
+    ## TODO: Check all patches present in patch directory are
+    ## mentioned in patch history; having an orphaned patch only gives
+    ## a warning.
+
+    ## TODO: Check cached data is consistent with data reconstructed
+    ## from scratch.
+
+    ## TODO: Check no control files are versioned.
+
+    ## TODO: Check that the before-hash of each file in a later
+    ## revision matches the after-hash in the previous revision to
+    ## touch it.
+
+
+def check_inventory():
+    mutter("checking inventory file and ids...")
+    seen_ids = Set()
+    seen_names = Set()
+    
+    for l in controlfile('inventory').readlines():
+        parts = l.split()
+        if len(parts) != 2:
+            bailout("malformed inventory line: " + `l`)
+        file_id, name = parts
+        
+        if file_id in seen_ids:
+            bailout("duplicated file id " + file_id)
+        seen_ids.add(file_id)
+
+        if name in seen_names:
+            bailout("duplicated file name in inventory: " + quotefn(name))
+        seen_names.add(name)
+        
+        if is_control_file(name):
+            raise BzrError("control file %s present in inventory" % quotefn(name))
+
+
+def check_patches_exist():
+    """Check constraint of current version: all patches exist"""
+    mutter("checking all patches are present...")
+    for pid in revision_history():
+        read_patch_header(pid)
+
+
+def check_patch_chaining():
+    """Check ancestry of patches and history file is consistent"""
+    mutter("checking patch chaining...")
+    prev = None
+    for pid in revision_history():
+        log_prev = read_patch_header(pid).precursor
+        if log_prev != prev:
+            bailout("inconsistent precursor links on " + pid)
+        prev = pid
+
+
+def check_patch_uniqueness():
+    """Make sure no patch is listed twice in the history.
+
+    This should be implied by having correct ancestry but I'll check it
+    anyhow."""
+    mutter("checking history for duplicates...")
+    seen = Set()
+    for pid in revision_history():
+        if pid in seen:
+            bailout("patch " + pid + " appears twice in history")
+        seen.add(pid)
+        
+

=== added file 'bzrlib/diff.py'
--- bzrlib/diff.py	1970-01-01 00:00:00 +0000
+++ bzrlib/diff.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,143 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from sets import Set
+
+from trace import mutter
+
+
+
+
+
+
+def diff_trees(old_tree, new_tree):
+    """Compute diff between two trees.
+
+    They may be in different branches and may be working or historical
+    trees.
+
+    Yields a sequence of (state, id, old_name, new_name, kind).
+    Each filename and each id is listed only once.
+    """
+
+    ## TODO: Compare files before diffing; only mention those that have changed
+
+    ## TODO: Set nice names in the headers, maybe include diffstat
+
+    ## TODO: Perhaps make this a generator rather than using
+    ## a callback object?
+
+    ## TODO: Allow specifying a list of files to compare, rather than
+    ## doing the whole tree?  (Not urgent.)
+
+    ## TODO: Allow diffing any two inventories, not just the
+    ## current one against one.  We mgiht need to specify two
+    ## stores to look for the files if diffing two branches.  That
+    ## might imply this shouldn't be primarily a Branch method.
+
+    ## XXX: This doesn't report on unknown files; that can be done
+    ## from a separate method.
+
+    old_it = old_tree.list_files()
+    new_it = new_tree.list_files()
+
+    def next(it):
+        try:
+            return it.next()
+        except StopIteration:
+            return None
+
+    old_item = next(old_it)
+    new_item = next(new_it)
+
+    # We step through the two sorted iterators in parallel, trying to
+    # keep them lined up.
+
+    while (old_item != None) or (new_item != None):
+        # OK, we still have some remaining on both, but they may be
+        # out of step.        
+        if old_item != None:
+            old_name, old_class, old_kind, old_id = old_item
+        else:
+            old_name = None
+            
+        if new_item != None:
+            new_name, new_class, new_kind, new_id = new_item
+        else:
+            new_name = None
+
+        mutter("   diff pairwise %r" % (old_item,))
+        mutter("                 %r" % (new_item,))
+
+        if old_item:
+            # can't handle the old tree being a WorkingTree
+            assert old_class == 'V'
+
+        if new_item and (new_class != 'V'):
+            yield new_class, None, None, new_name, new_kind
+            new_item = next(new_it)
+        elif (not new_item) or (old_item and (old_name < new_name)):
+            mutter("     extra entry in old-tree sequence")
+            if new_tree.has_id(old_id):
+                # will be mentioned as renamed under new name
+                pass
+            else:
+                yield 'D', old_id, old_name, None, old_kind
+            old_item = next(old_it)
+        elif (not old_item) or (new_item and (new_name < old_name)):
+            mutter("     extra entry in new-tree sequence")
+            if old_tree.has_id(new_id):
+                yield 'R', new_id, old_tree.id2path(new_id), new_name, new_kind
+            else:
+                yield 'A', new_id, None, new_name, new_kind
+            new_item = next(new_it)
+        elif old_id != new_id:
+            assert old_name == new_name
+            # both trees have a file of this name, but it is not the
+            # same file.  in other words, the old filename has been
+            # overwritten by either a newly-added or a renamed file.
+            # (should we return something about the overwritten file?)
+            if old_tree.has_id(new_id):
+                # renaming, overlying a deleted file
+                yield 'R', new_id, old_tree.id2path(new_id), new_name, new_kind
+            else:
+                yield 'A', new_id, None, new_name, new_kind
+
+            new_item = next(new_it)
+            old_item = next(old_it)
+        else:
+            assert old_id == new_id
+            assert old_name == new_name
+            assert old_kind == new_kind
+
+            if old_kind == 'directory':
+                yield '.', new_id, old_name, new_name, new_kind
+            elif old_tree.get_file_size(old_id) != new_tree.get_file_size(old_id):
+                mutter("    file size has changed, must be different")
+                yield 'M', new_id, old_name, new_name, new_kind
+            elif old_tree.get_file_sha1(old_id) == new_tree.get_file_sha1(old_id):
+                mutter("      SHA1 indicates they're identical")
+                ## assert compare_files(old_tree.get_file(i), new_tree.get_file(i))
+                yield '.', new_id, old_name, new_name, new_kind
+            else:
+                mutter("      quick compare shows different")
+                yield 'M', new_id, old_name, new_name, new_kind
+
+            new_item = next(new_it)
+            old_item = next(old_it)
+
+

=== added file 'bzrlib/errors.py'
--- bzrlib/errors.py	1970-01-01 00:00:00 +0000
+++ bzrlib/errors.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,39 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+__copyright__ = "Copyright (C) 2005 Canonical Ltd."
+__author__ = "Martin Pool <mbp@canonical.com>"
+
+
+######################################################################
+# exceptions 
+class BzrError(StandardError):
+    pass
+
+class BzrCheckError(BzrError):
+    pass
+
+
+
+
+def bailout(msg, explanation=[]):
+    ex = BzrError(msg, explanation)
+    import trace
+    trace._tracefile.write('* raising %s\n' % ex)
+    raise ex
+

=== added file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/inventory.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,498 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Inventories map files to their name in a revision."""
+
+
+__copyright__ = "Copyright (C) 2005 Canonical Ltd."
+__author__ = "Martin Pool <mbp@canonical.com>"
+
+import sys, os.path, types
+from sets import Set
+
+from xml import XMLMixin
+from ElementTree import ElementTree, Element
+from errors import bailout
+from osutils import uuid, quotefn, splitpath, joinpath, appendpath
+from trace import mutter
+
+class InventoryEntry(XMLMixin):
+    """Description of a versioned file.
+
+    An InventoryEntry has the following fields, which are also
+    present in the XML inventory-entry element:
+
+    * *file_id*
+    * *name*: (only the basename within the directory, must not
+      contain slashes)
+    * *kind*: "directory" or "file"
+    * *directory_id*: (if absent/null means the branch root directory)
+    * *text_sha1*: only for files
+    * *text_size*: in bytes, only for files 
+    * *text_id*: identifier for the text version, only for files
+
+    InventoryEntries can also exist inside a WorkingTree
+    inventory, in which case they are not yet bound to a
+    particular revision of the file.  In that case the text_sha1,
+    text_size and text_id are absent.
+
+
+    >>> i = Inventory()
+    >>> i.path2id('')
+    >>> i.add(InventoryEntry('123', 'src', kind='directory'))
+    >>> i.add(InventoryEntry('2323', 'hello.c', parent_id='123'))
+    >>> for j in i.iter_entries():
+    ...   print j
+    ... 
+    ('src', InventoryEntry('123', 'src', kind='directory', parent_id=None))
+    ('src/hello.c', InventoryEntry('2323', 'hello.c', kind='file', parent_id='123'))
+    >>> i.add(InventoryEntry('2323', 'bye.c', parent_id='123'))
+    Traceback (most recent call last):
+    ...
+    BzrError: ('inventory already contains entry with id {2323}', [])
+    >>> i.add(InventoryEntry('2324', 'bye.c', parent_id='123'))
+    >>> i.add(InventoryEntry('2325', 'wibble', parent_id='123', kind='directory'))
+    >>> i.path2id('src/wibble')
+    '2325'
+    >>> '2325' in i
+    True
+    >>> i.add(InventoryEntry('2326', 'wibble.c', parent_id='2325'))
+    >>> i['2326']
+    InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
+    >>> for j in i.iter_entries():
+    ...     print j[0]
+    ...     assert i.path2id(j[0])
+    ... 
+    src
+    src/bye.c
+    src/hello.c
+    src/wibble
+    src/wibble/wibble.c
+    >>> i.id2path('2326')
+    'src/wibble/wibble.c'
+
+    :todo: Maybe also keep the full path of the entry, and the children?
+           But those depend on its position within a particular inventory, and
+           it would be nice not to need to hold the backpointer here.
+    """
+    def __init__(self, file_id, name, kind='file', text_id=None,
+                 parent_id=None):
+        """Create an InventoryEntry
+        
+        The filename must be a single component, relative to the
+        parent directory; it cannot be a whole path or relative name.
+
+        >>> e = InventoryEntry('123', 'hello.c')
+        >>> e.name
+        'hello.c'
+        >>> e.file_id
+        '123'
+        >>> e = InventoryEntry('123', 'src/hello.c')
+        Traceback (most recent call last):
+        BzrError: ("InventoryEntry name is not a simple filename: 'src/hello.c'", [])
+        """
+        
+        if len(splitpath(name)) != 1:
+            bailout('InventoryEntry name is not a simple filename: %r'
+                    % name)
+        
+        self.file_id = file_id
+        self.name = name
+        assert kind in ['file', 'directory']
+        self.kind = kind
+        self.text_id = text_id
+        self.parent_id = parent_id
+        self.text_sha1 = None
+        self.text_size = None
+
+
+    def copy(self):
+        other = InventoryEntry(self.file_id, self.name, self.kind,
+                               self.text_id, self.parent_id)
+        other.text_sha1 = self.text_sha1
+        other.text_size = self.text_size
+        return other
+
+
+    def __repr__(self):
+        return ("%s(%r, %r, kind=%r, parent_id=%r)"
+                % (self.__class__.__name__,
+                   self.file_id,
+                   self.name,
+                   self.kind,
+                   self.parent_id))
+
+    
+    def to_element(self):
+        """Convert to XML element"""
+        e = Element('entry')
+
+        e.set('name', self.name)
+        e.set('file_id', self.file_id)
+        e.set('kind', self.kind)
+
+        if self.text_size is not None:
+            e.set('text_size', '%d' % self.text_size)
+            
+        for f in ['text_id', 'text_sha1', 'parent_id']:
+            v = getattr(self, f)
+            if v is not None:
+                e.set(f, v)
+
+        e.tail = '\n'
+            
+        return e
+
+
+    def from_element(cls, elt):
+        assert elt.tag == 'entry'
+        self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'))
+        self.text_id = elt.get('text_id')
+        self.text_sha1 = elt.get('text_sha1')
+        self.parent_id = elt.get('parent_id')
+        
+        ## mutter("read inventoryentry: %r" % (elt.attrib))
+
+        v = elt.get('text_size')
+        self.text_size = v and int(v)
+
+        return self
+            
+
+    from_element = classmethod(from_element)
+
+    def __cmp__(self, other):
+        if self is other:
+            return 0
+        if not isinstance(other, InventoryEntry):
+            return NotImplemented
+
+        return cmp(self.file_id, other.file_id) \
+               or cmp(self.name, other.name) \
+               or cmp(self.text_sha1, other.text_sha1) \
+               or cmp(self.text_size, other.text_size) \
+               or cmp(self.text_id, other.text_id) \
+               or cmp(self.parent_id, other.parent_id) \
+               or cmp(self.kind, other.kind)
+
+
+
+class Inventory(XMLMixin):
+    """Inventory of versioned files in a tree.
+
+    An Inventory acts like a set of InventoryEntry items.  You can
+    also look files up by their file_id or name.
+    
+    May be read from and written to a metadata file in a tree.  To
+    manipulate the inventory (for example to add a file), it is read
+    in, modified, and then written back out.
+
+    The inventory represents a typical unix file tree, with
+    directories containing files and subdirectories.  We never store
+    the full path to a file, because renaming a directory implicitly
+    moves all of its contents.  This class internally maintains a
+    lookup tree that allows the children under a directory to be
+    returned quickly.
+
+    InventoryEntry objects must not be modified after they are
+    inserted.
+
+    >>> inv = Inventory()
+    >>> inv.write_xml(sys.stdout)
+    <inventory>
+    </inventory>
+    >>> inv.add(InventoryEntry('123-123', 'hello.c'))
+    >>> inv['123-123'].name
+    'hello.c'
+    >>> for file_id in inv: print file_id
+    ...
+    123-123
+
+    May be treated as an iterator or set to look up file ids:
+    
+    >>> bool(inv.path2id('hello.c'))
+    True
+    >>> '123-123' in inv
+    True
+
+    May also look up by name:
+
+    >>> [x[0] for x in inv.iter_entries()]
+    ['hello.c']
+    
+    >>> inv.write_xml(sys.stdout)
+    <inventory>
+    <entry file_id="123-123" kind="file" name="hello.c" />
+    </inventory>
+
+    """
+
+    ## TODO: Clear up handling of files in subdirectories; we probably
+    ## do want to be able to just look them up by name but this
+    ## probably means gradually walking down the path, looking up as we go.
+
+    ## TODO: Make sure only canonical filenames are stored.
+
+    ## TODO: Do something sensible about the possible collisions on
+    ## case-losing filesystems.  Perhaps we should just always forbid
+    ## such collisions.
+
+    ## _tree should probably just be stored as
+    ## InventoryEntry._children on each directory.
+
+    def __init__(self):
+        """Create or read an inventory.
+
+        If a working directory is specified, the inventory is read
+        from there.  If the file is specified, read from that. If not,
+        the inventory is created empty.
+        """
+        self._byid = dict()
+
+        # _tree is indexed by parent_id; at each level a map from name
+        # to ie.  The None entry is the root.
+        self._tree = {None: {}}
+
+
+    def __iter__(self):
+        return iter(self._byid)
+
+
+    def __len__(self):
+        """Returns number of entries."""
+        return len(self._byid)
+
+
+    def iter_entries(self, parent_id=None):
+        """Return (path, entry) pairs, in order by name."""
+        kids = self._tree[parent_id].items()
+        kids.sort()
+        for name, ie in kids:
+            yield name, ie
+            if ie.kind == 'directory':
+                for cn, cie in self.iter_entries(parent_id=ie.file_id):
+                    yield joinpath([name, cn]), cie
+
+
+    def directories(self, include_root=True):
+        """Return (path, entry) pairs for all directories.
+        """
+        if include_root:
+            yield '', None
+        for path, entry in self.iter_entries():
+            if entry.kind == 'directory':
+                yield path, entry
+        
+
+
+    def children(self, parent_id):
+        """Return entries that are direct children of parent_id."""
+        return self._tree[parent_id]
+                    
+
+
+    # TODO: return all paths and entries
+
+
+    def __contains__(self, file_id):
+        """True if this entry contains a file with given id.
+
+        >>> inv = Inventory()
+        >>> inv.add(InventoryEntry('123', 'foo.c'))
+        >>> '123' in inv
+        True
+        >>> '456' in inv
+        False
+        """
+        return file_id in self._byid
+
+
+    def __getitem__(self, file_id):
+        """Return the entry for given file_id.
+
+        >>> inv = Inventory()
+        >>> inv.add(InventoryEntry('123123', 'hello.c'))
+        >>> inv['123123'].name
+        'hello.c'
+        """
+        return self._byid[file_id]
+
+
+    def add(self, entry):
+        """Add entry to inventory.
+
+        To add  a file to a branch ready to be committed, use Branch.add,
+        which calls this."""
+        if entry.file_id in self:
+            bailout("inventory already contains entry with id {%s}" % entry.file_id)
+
+        if entry.parent_id != None:
+            if entry.parent_id not in self:
+                bailout("parent_id %s of new entry not found in inventory"
+                        % entry.parent_id)
+            
+        if self._tree[entry.parent_id].has_key(entry.name):
+            bailout("%s is already versioned"
+                    % appendpath(self.id2path(entry.parent_id), entry.name))
+
+        self._byid[entry.file_id] = entry
+        self._tree[entry.parent_id][entry.name] = entry
+
+        if entry.kind == 'directory':
+            self._tree[entry.file_id] = {}
+
+
+    def __delitem__(self, file_id):
+        """Remove entry by id.
+
+        >>> inv = Inventory()
+        >>> inv.add(InventoryEntry('123', 'foo.c'))
+        >>> '123' in inv
+        True
+        >>> del inv['123']
+        >>> '123' in inv
+        False
+        """
+        ie = self[file_id]
+
+        assert self._tree[ie.parent_id][ie.name] == ie
+        
+        # TODO: Test deleting all children; maybe hoist to a separate
+        # deltree method?
+        if ie.kind == 'directory':
+            for cie in self._tree[file_id].values():
+                del self[cie.file_id]
+            del self._tree[file_id]
+
+        del self._byid[file_id]
+        del self._tree[ie.parent_id][ie.name]
+
+
+    def id_set(self):
+        return Set(self._byid)
+
+
+    def to_element(self):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        for path, ie in self.iter_entries():
+            e.append(ie.to_element())
+        return e
+    
+
+    def from_element(cls, elt):
+        """Construct from XML Element
+
+        >>> inv = Inventory()
+        >>> inv.add(InventoryEntry('foo.c-123981239', 'foo.c'))
+        >>> elt = inv.to_element()
+        >>> inv2 = Inventory.from_element(elt)
+        >>> inv2 == inv
+        True
+        """
+        assert elt.tag == 'inventory'
+        o = cls()
+        for e in elt:
+            o.add(InventoryEntry.from_element(e))
+        return o
+        
+    from_element = classmethod(from_element)
+
+
+    def __cmp__(self, other):
+        """Compare two sets by comparing their contents.
+
+        >>> i1 = Inventory()
+        >>> i2 = Inventory()
+        >>> i1 == i2
+        True
+        >>> i1.add(InventoryEntry('123', 'foo'))
+        >>> i1 == i2
+        False
+        >>> i2.add(InventoryEntry('123', 'foo'))
+        >>> i1 == i2
+        True
+        """
+        if self is other:
+            return 0
+        
+        if not isinstance(other, Inventory):
+            return NotImplemented
+
+        if self.id_set() ^ other.id_set():
+            return 1
+
+        for file_id in self._byid:
+            c = cmp(self[file_id], other[file_id])
+            if c: return c
+
+        return 0
+
+
+    def id2path(self, file_id):
+        """Return as a list the path to file_id."""
+        p = []
+        while file_id != None:
+            ie = self[file_id]
+            p = [ie.name] + p
+            file_id = ie.parent_id
+        return joinpath(p)
+            
+
+
+    def path2id(self, name):
+        """Walk down through directories to return entry of last component.
+
+        names may be either a list of path components, or a single
+        string, in which case it is automatically split.
+
+        This returns the entry of the last component in the path,
+        which may be either a file or a directory.
+        """
+        assert isinstance(name, types.StringTypes)
+
+        parent_id = None
+        for f in splitpath(name):
+            try:
+                cie = self._tree[parent_id][f]
+                assert cie.name == f
+                parent_id = cie.file_id
+            except KeyError:
+                # or raise an error?
+                return None
+
+        return parent_id
+
+
+    def get_child(self, parent_id, child_name):
+        return self._tree[parent_id].get(child_name)
+
+
+    def has_filename(self, names):
+        return bool(self.path2id(names))
+
+
+    def has_id(self, file_id):
+        assert isinstance(file_id, str)
+        return self._byid.has_key(file_id)
+
+
+
+if __name__ == '__main__':
+    import doctest, inventory
+    doctest.testmod(inventory)

=== added file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	1970-01-01 00:00:00 +0000
+++ bzrlib/osutils.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,259 @@
+# Bazaar-NG -- distributed version control
+
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os, types, re, time, types
+from stat import S_ISREG, S_ISDIR, ST_MODE, ST_SIZE
+
+from errors import bailout
+
+def make_readonly(filename):
+    """Make a filename read-only."""
+    # TODO: probably needs to be fixed for windows
+    mod = os.stat(filename).st_mode
+    mod = mod & 0777555
+    os.chmod(filename, mod)
+
+
+def make_writable(filename):
+    mod = os.stat(filename).st_mode
+    mod = mod | 0200
+    os.chmod(filename, mod)
+
+
+_QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/_~-])')
+def quotefn(f):
+    """Return shell-quoted filename"""
+    ## We could be a bit more terse by using double-quotes etc
+    f = _QUOTE_RE.sub(r'\\\1', f)
+    if f[0] == '~':
+        f[0:1] = r'\~' 
+    return f
+
+
+def file_kind(f):
+    mode = os.lstat(f)[ST_MODE]
+    if S_ISREG(mode):
+        return 'file'
+    elif S_ISDIR(mode):
+        return 'directory'
+    else:
+        bailout("can't handle file kind of %r" % fp)
+
+
+
+def isdir(f):
+    """True if f is an accessible directory."""
+    try:
+        return S_ISDIR(os.lstat(f)[ST_MODE])
+    except OSError:
+        return False
+
+
+
+def isfile(f):
+    """True if f is a regular file."""
+    try:
+        return S_ISREG(os.lstat(f)[ST_MODE])
+    except OSError:
+        return False
+
+
+def pumpfile(fromfile, tofile):
+    """Copy contents of one file to another."""
+    tofile.write(fromfile.read())
+
+
+def uuid():
+    """Return a new UUID"""
+    
+    ## XXX: Could alternatively read /proc/sys/kernel/random/uuid on
+    ## Linux, but we need something portable for other systems;
+    ## preferably an implementation in Python.
+    bailout('uuids not allowed!')
+    return chomp(os.popen('uuidgen').readline())
+
+def chomp(s):
+    if s and (s[-1] == '\n'):
+        return s[:-1]
+    else:
+        return s
+
+
+def sha_file(f):
+    import sha
+    ## TODO: Maybe read in chunks to handle big files
+    if hasattr(f, 'tell'):
+        assert f.tell() == 0
+    s = sha.new()
+    s.update(f.read())
+    return s.hexdigest()
+
+
+def sha_string(f):
+    import sha
+    s = sha.new()
+    s.update(f)
+    return s.hexdigest()
+
+
+
+def username():
+    """Return email-style username.
+
+    Something similar to 'Martin Pool <mbp@sourcefrog.net>'
+
+    :todo: Check it's reasonably well-formed.
+
+    :todo: Allow taking it from a dotfile to help people on windows
+           who can't easily set variables.
+
+    :todo: Cope without pwd module, which is only on unix. 
+    """
+    e = os.environ.get('BZREMAIL') or os.environ.get('EMAIL')
+    if e: return e
+
+    import socket
+    
+    try:
+        import pwd
+        uid = os.getuid()
+        w = pwd.getpwuid(uid)
+        realname, junk = w.pw_gecos.split(',', 1)
+        return '%s <%s@%s>' % (realname, w.pw_name, socket.getfqdn())
+    except ImportError:
+        pass
+
+    import getpass, socket
+    return '<%s@%s>' % (getpass.getuser(), socket.getfqdn())
+
+
+def user_email():
+    """Return just the email component of a username."""
+    e = os.environ.get('BZREMAIL') or os.environ.get('EMAIL')
+    if e:
+        import re
+        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
+        if not m:
+            bailout('%r is not a reasonable email address' % e)
+        return m.group(0)
+
+
+    import getpass, socket
+    return '%s@%s' % (getpass.getuser(), socket.getfqdn())
+
+    
+
+
+def compare_files(a, b):
+    """Returns true if equal in contents"""
+    # TODO: don't read the whole thing in one go.
+    result = a.read() == b.read()
+    return result
+
+
+
+def format_date(t, inutc=False):
+    ## TODO: Perhaps a global option to use either universal or local time?
+    ## Or perhaps just let people set $TZ?
+    import time
+    
+    assert isinstance(t, float)
+    
+    if inutc:
+        tt = time.gmtime(t)
+        zonename = 'UTC'
+        offset = 0
+    else:
+        tt = time.localtime(t)
+        if time.daylight:
+            zonename = time.tzname[1]
+            offset = - time.altzone
+        else:
+            zonename = time.tzname[0]
+            offset = - time.timezone
+            
+    return (time.strftime("%a %Y-%m-%d %H:%M:%S", tt)
+            + ' ' + zonename + ' '
+            + '%+03d%02d' % (offset / 3600, (offset / 60) % 60))
+
+
+def compact_date(when):
+    return time.strftime('%Y%m%d%H%M%S', time.gmtime(when))
+    
+
+
+def filesize(f):
+    """Return size of given open file."""
+    return os.fstat(f.fileno())[ST_SIZE]
+
+
+if hasattr(os, 'urandom'): # python 2.4 and later
+    rand_bytes = os.urandom
+else:
+    # FIXME: No good on non-Linux
+    _rand_file = file('/dev/urandom', 'rb')
+    rand_bytes = _rand_file.read
+
+
+## TODO: We could later have path objects that remember their list
+## decomposition (might be too tricksy though.)
+
+def splitpath(p):
+    """Turn string into list of parts.
+
+    >>> splitpath('a')
+    ['a']
+    >>> splitpath('a/b')
+    ['a', 'b']
+    >>> splitpath('a/./b')
+    ['a', 'b']
+    >>> splitpath('a/.b')
+    ['a', '.b']
+    >>> splitpath('a/../b')
+    Traceback (most recent call last):
+    ...
+    BzrError: ("sorry, '..' not allowed in path", [])
+    """
+    assert isinstance(p, types.StringTypes)
+    ps = [f for f in p.split('/') if f != '.']
+    for f in ps:
+        if f == '..':
+            bailout("sorry, %r not allowed in path" % f)
+    return ps
+
+def joinpath(p):
+    assert isinstance(p, list)
+    for f in p:
+        if (f == '..') or (f is None) or (f == ''):
+            bailout("sorry, %r not allowed in path" % f)
+    return '/'.join(p)
+
+
+def appendpath(p1, p2):
+    if p1 == '':
+        return p2
+    else:
+        return p1 + '/' + p2
+    
+
+def extern_command(cmd, ignore_errors = False):
+    mutter('external command: %s' % `cmd`)
+    if os.system(cmd):
+        if not ignore_errors:
+            bailout('command failed')
+

=== added file 'bzrlib/revision.py'
--- bzrlib/revision.py	1970-01-01 00:00:00 +0000
+++ bzrlib/revision.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,72 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+from xml import XMLMixin
+from ElementTree import Element, ElementTree, SubElement
+
+class Revision(XMLMixin):
+    """Single revision on a branch.
+
+    Revisions may know their revision_hash, but only once they've been
+    written out.  This is not stored because you cannot write the hash
+    into the file it describes.
+
+    :todo: Perhaps make predecessor be a child element, not an attribute?
+    """
+    def __init__(self, **args):
+        self.inventory_id = None
+        self.revision_id = None
+        self.timestamp = None
+        self.message = None
+        self.__dict__.update(args)
+
+
+    def __repr__(self):
+        if self.revision_id:
+            return "<Revision id %s>" % self.revision_id
+
+        
+    def to_element(self):
+        root = Element('changeset',
+                       committer = self.committer,
+                       timestamp = '%f' % self.timestamp,
+                       revision_id = self.revision_id,
+                       inventory_id = self.inventory_id)
+        if self.precursor:
+            root.set('precursor', self.precursor)
+        root.text = '\n'
+        
+        msg = SubElement(root, 'message')
+        msg.text = self.message
+        msg.tail = '\n'
+
+        return root
+
+    def from_element(cls, root):
+        cs = cls(committer = root.get('committer'),
+                 timestamp = float(root.get('timestamp')),
+                 precursor = root.get('precursor'),
+                 revision_id = root.get('revision_id'),
+                 inventory_id = root.get('inventory_id'))
+
+        cs.message = root.findtext('message') # text of <message>
+        return cs
+
+    from_element = classmethod(from_element)
+

=== added file 'bzrlib/store.py'
--- bzrlib/store.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,147 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Stores are the main data-storage mechanism for Bazaar-NG.
+
+A store is a simple write-once container indexed by a universally
+unique ID, which is typically the SHA-1 of the content."""
+
+__copyright__ = "Copyright (C) 2005 Canonical Ltd."
+__author__ = "Martin Pool <mbp@canonical.com>"
+
+import os, tempfile, types, osutils
+from StringIO import StringIO
+from trace import mutter
+
+
+######################################################################
+# stores
+
+class StoreError(Exception):
+    pass
+
+
+class ImmutableStore:
+    """Store that holds files indexed by unique names.
+
+    Files can be added, but not modified once they are in.  Typically
+    the hash is used as the name, or something else known to be unique,
+    such as a UUID.
+
+    >>> st = ImmutableScratchStore()
+
+    >>> st.add(StringIO('hello'), 'aa')
+    >>> 'aa' in st
+    True
+    >>> 'foo' in st
+    False
+
+    You are not allowed to add an id that is already present.
+
+    Entries can be retrieved as files, which may then be read.
+
+    >>> st.add(StringIO('goodbye'), '123123')
+    >>> st['123123'].read()
+    'goodbye'
+
+    :todo: Atomic add by writing to a temporary file and renaming.
+
+    :todo: Perhaps automatically transform to/from XML in a method?
+           Would just need to tell the constructor what class to
+           use...
+
+    :todo: Even within a simple disk store like this, we could
+           gzip the files.  But since many are less than one disk
+           block, that might not help a lot.
+
+    """
+
+    def __init__(self, basedir):
+        """ImmutableStore constructor."""
+        self._basedir = basedir
+
+    def _path(self, id):
+        return os.path.join(self._basedir, id)
+
+    def __repr__(self):
+        return "%s(%r)" % (self.__class__.__name__, self._basedir)
+
+    def add(self, f, fileid):
+        """Add contents of a file into the store.
+
+        :param f: An open file, or file-like object."""
+        # FIXME: Only works on smallish files
+        # TODO: Can be optimized by copying at the same time as
+        # computing the sum.
+        mutter("add store entry %r" % (fileid))
+        if isinstance(f, types.StringTypes):
+            content = f
+        else:
+            content = f.read()
+        if fileid not in self:
+            filename = self._path(fileid)
+            f = file(filename, 'wb')
+            f.write(content)
+            f.flush()
+            os.fsync(f.fileno())
+            f.close()
+            osutils.make_readonly(filename)
+
+
+    def __contains__(self, fileid):
+        """"""
+        return os.access(self._path(fileid), os.R_OK)
+
+
+    def __iter__(self):
+        return iter(os.listdir(self._basedir))
+
+    def __getitem__(self, fileid):
+        """Returns a file reading from a particular entry."""
+        return file(self._path(fileid), 'rb')
+
+    def delete_all(self):
+        for fileid in self:
+            self.delete(fileid)
+
+    def delete(self, fileid):
+        """Remove nominated store entry.
+
+        Most stores will be add-only."""
+        filename = self._path(fileid)
+        ## osutils.make_writable(filename)
+        os.remove(filename)
+
+    def destroy(self):
+        """Remove store; only allowed if it is empty."""
+        os.rmdir(self._basedir)
+        mutter("%r destroyed" % self)
+
+
+
+class ImmutableScratchStore(ImmutableStore):
+    """Self-destructing test subclass of ImmutableStore.
+
+    The Store only exists for the lifetime of the Python object.
+    Obviously you should not put anything precious in it.
+    """
+    def __init__(self):
+        ImmutableStore.__init__(self, tempfile.mkdtemp())
+
+    def __del__(self):
+        self.delete_all()
+        self.destroy()

=== added file 'bzrlib/tests.py'
--- bzrlib/tests.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,172 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+# XXX: We might prefer these to be in a text file rather than Python
+# source, but that only works in doctest from Python 2.4 and later,
+# which is not present in Warty.
+
+r"""
+Bazaar-NG test cases
+********************
+
+These are run by ``bzr.doctest``.
+
+>>> import bzr, bzrlib, os
+>>> bzr.cmd_rocks()
+it sure does!
+
+Hey, nice place to begin.
+
+The basic object is a Branch.  We have a special helper class
+ScratchBranch that automatically makes a directory and cleans itself
+up, but is in other respects identical.
+
+ScratchBranches are initially empty:
+
+>>> b = bzr.ScratchBranch()
+>>> b.show_status()
+
+New files in that directory are, it is initially unknown:
+
+>>> file(b.base + '/hello.c', 'wt').write('int main() {}')
+>>> b.show_status()
+?       hello.c
+
+That's not quite true; some files (like editor backups) are ignored by
+default:
+
+>>> file(b.base + '/hello.c~', 'wt').write('int main() {}')
+>>> b.show_status()
+?       hello.c
+>>> list(b.unknowns())
+['hello.c']
+
+The ``add`` command marks a file to be added in the next revision:
+
+>>> b.add('hello.c')
+>>> b.show_status()
+A       hello.c
+
+You can also add files that otherwise would be ignored.  The ignore
+patterns only apply to files that would be otherwise unknown, so they
+have no effect once it's added.
+
+>>> b.add('hello.c~')
+>>> b.show_status()
+A       hello.c
+A       hello.c~
+
+It is an error to add a file that isn't present in the working copy:
+
+  >>> b.add('nothere')
+  Traceback (most recent call last):
+  ...
+  BzrError: ('cannot add: not a regular file or directory: nothere', [])
+
+If we add a file and then change our mind, we can either revert it or
+remove the file.  If we revert, we are left with the working copy (in
+either I or ? state).  If we remove, the working copy is gone.  Let's
+do that to the backup, presumably added accidentally.
+
+  >>> b.remove('hello.c~')
+  >>> b.show_status()
+  A       hello.c
+
+Now to commit, creating a new revision.  (Fake the date and name for
+reproducibility.)
+
+  >>> b.commit('start hello world', timestamp=0, committer='foo@nowhere')
+  >>> b.show_status()
+  >>> b.show_status(show_all=True)
+  .       hello.c
+  I       hello.c~
+
+
+We can look back at history
+
+  >>> r = b.get_revision(b.lookup_revision(1))
+  >>> r.message
+  'start hello world'
+  >>> b.write_log(utc=True)
+  ----------------------------------------
+  revno: 1
+  committer: foo@nowhere
+  timestamp: Thu 1970-01-01 00:00:00 UTC +0000
+  message:
+    start hello world
+
+(The other fields will be a bit unpredictable, depending on who ran
+this test and when.)
+
+As of 2005-02-21, we can also add subdirectories to the revision!
+
+  >>> os.mkdir(b.base + "/lib")
+  >>> b.show_status()
+  ?       lib/
+  >>> b.add('lib')
+  >>> b.show_status()
+  A       lib/
+  >>> b.commit('add subdir')
+  >>> b.show_status()
+  >>> b.show_status(show_all=True)
+  .       hello.c
+  I       hello.c~
+  .       lib/
+
+and we can also add files within subdirectories:
+
+  >>> file(b.base + '/lib/hello', 'w').write('hello!\n')
+  >>> b.show_status()
+  ?       lib/hello
+  
+  
+Tests for adding subdirectories, etc.
+
+    >>> b = bzrlib.branch.ScratchBranch()
+    >>> os.mkdir(b._rel('d1'))
+    >>> os.mkdir(b._rel('d2'))
+    >>> os.mkdir(b._rel('d2/d3'))
+    >>> list(b.working_tree().unknowns())
+    ['d1', 'd2']
+
+Create some files, but they're not seen as unknown yet:
+
+    >>> file(b._rel('d1/f1'), 'w').close()
+    >>> file(b._rel('d2/f2'), 'w').close()
+    >>> file(b._rel('d2/f3'), 'w').close()
+    >>> [v[0] for v in b.inventory.directories()]
+    ['']
+    >>> list(b.working_tree().unknowns())
+    ['d1', 'd2']
+
+Adding a directory, and we see the file underneath:
+    
+    >>> b.add('d1')
+    >>> [v[0] for v in b.inventory.directories()]
+    ['', 'd1']
+    >>> list(b.working_tree().unknowns())
+    ['d1/f1', 'd2']
+    >>> # d2 comes first because it's in the top directory
+
+    >>> b.add('d2')
+    >>> b.commit('add some stuff')
+    >>> list(b.working_tree().unknowns())
+    ['d1/f1', 'd2/d3', 'd2/f2', 'd2/f3']
+
+"""

=== added file 'bzrlib/textui.py'
--- bzrlib/textui.py	1970-01-01 00:00:00 +0000
+++ bzrlib/textui.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,30 @@
+# Bazaar-NG -- distributed version control
+
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+def show_status(state, kind, name):
+    if kind == 'directory':
+        kind_ch = '/'
+    else:
+        assert kind == 'file'
+        kind_ch = ''
+
+    assert len(state) == 1
+        
+    print state + '       ' + name + kind_ch
+    

=== added file 'bzrlib/trace.py'
--- bzrlib/trace.py	1970-01-01 00:00:00 +0000
+++ bzrlib/trace.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,65 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+__copyright__ = "Copyright (C) 2005 Canonical Ltd."
+__author__ = "Martin Pool <mbp@canonical.com>"
+
+
+import sys
+
+######################################################################
+# messages and logging
+
+# Messages are always written to here, so that we have some
+# information if something goes wrong.  In a future version this
+# file will be removed on successful completion.
+_tracefile = file('.bzr.log', 'at')
+
+## TODO: If --verbose is given then write to both stderr and
+## _tracefile; perhaps replace _tracefile with a tee thing.
+
+# used to have % (os.environ['USER'], time.time(), os.getpid()), 'w')
+
+
+# If false, notes also go to stdout; should replace this with --silent
+# at some point.
+silent = False
+
+verbose = False
+
+
+def mutter(msg):
+    _tracefile.write(msg)
+    _tracefile.write('\n')
+    _tracefile.flush()
+    if verbose:
+        sys.stderr.write('- ' + msg + '\n')
+
+
+def note(msg):
+    b = '* ' + str(msg) + '\n'
+    if not silent:
+        sys.stderr.write(b)
+    _tracefile.write(b)
+    _tracefile.flush()
+
+
+def log_error(msg):
+    sys.stderr.write(msg)
+    _tracefile.write(msg)
+    _tracefile.flush()

=== added file 'bzrlib/tree.py'
--- bzrlib/tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tree.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,382 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tree classes, representing directory at point in time.
+"""
+
+from sets import Set
+import os.path, os, fnmatch
+
+from inventory import Inventory
+from trace import mutter, note
+from osutils import pumpfile, compare_files, filesize, quotefn, sha_file, \
+     joinpath, splitpath, appendpath, isdir, isfile, file_kind
+from errors import bailout
+import branch
+from stat import S_ISREG, S_ISDIR, ST_MODE, ST_SIZE
+
+import bzrlib
+
+class Tree:
+    """Abstract file tree.
+
+    There are several subclasses:
+    
+    * `WorkingTree` exists as files on disk editable by the user.
+
+    * `RevisionTree` is a tree as recorded at some point in the past.
+
+    * `EmptyTree`
+
+    Trees contain an `Inventory` object, and also know how to retrieve
+    file texts mentioned in the inventory, either from a working
+    directory or from a store.
+
+    It is possible for trees to contain files that are not described
+    in their inventory or vice versa; for this use `filenames()`.
+
+    Trees can be compared, etc, regardless of whether they are working
+    trees or versioned trees.
+    """
+    
+    def get_file(self, file_id):
+        """Return an open file-like object for given file id."""
+        raise NotImplementedError()
+
+    def has_filename(self, filename):
+        """True if the tree has given filename."""
+        raise NotImplementedError()
+
+    def has_id(self, file_id):
+        return self.inventory.has_id(file_id)
+
+    def id_set(self):
+        """Return set of all ids in this tree."""
+        return self.inventory.id_set()
+
+    def id2path(self, file_id):
+        return self.inventory.id2path(file_id)
+
+    def _get_inventory(self):
+        return self._inventory
+
+    inventory = property(_get_inventory,
+                         doc="Inventory of this Tree")
+
+    def _check_retrieved(self, ie, f):
+        # TODO: Test this check by damaging the store?
+        if ie.text_size is not None:
+            fs = filesize(f)
+            if fs != ie.text_size:
+                bailout("mismatched size for file %r in %r" % (ie.file_id, self._store),
+                        ["inventory expects %d bytes" % ie.text_size,
+                         "file is actually %d bytes" % fs,
+                         "store is probably damaged/corrupt"])
+
+        f_hash = sha_file(f)
+        f.seek(0)
+        if ie.text_sha1 != f_hash:
+            bailout("wrong SHA-1 for file %r in %r" % (ie.file_id, self._store),
+                    ["inventory expects %s" % ie.text_sha1,
+                     "file is actually %s" % f_hash,
+                     "store is probably damaged/corrupt"])
+
+
+    def export(self, dest):
+        """Export this tree to a new directory.
+
+        `dest` should not exist, and will be created holding the
+        contents of this tree.
+
+        :todo: To handle subdirectories we need to create the
+               directories first.
+
+        :note: If the export fails, the destination directory will be
+               left in a half-assed state.
+        """
+        os.mkdir(dest)
+        mutter('export version %r' % self)
+        inv = self.inventory
+        for dp, ie in inv.iter_entries():
+            kind = ie.kind
+            fullpath = appendpath(dest, dp)
+            if kind == 'directory':
+                os.mkdir(fullpath)
+            elif kind == 'file':
+                pumpfile(self.get_file(ie.file_id), file(fullpath, 'wb'))
+            else:
+                bailout("don't know how to export {%s} of kind %r", fid, kind)
+            mutter("  export {%s} kind %s to %s" % (ie.file_id, kind, fullpath))
+
+
+
+class WorkingTree(Tree):
+    """Working copy tree.
+
+    The inventory is held in the `Branch` working-inventory, and the
+    files are in a directory on disk.
+
+    It is possible for a `WorkingTree` to have a filename which is
+    not listed in the Inventory and vice versa.
+    """
+    def __init__(self, basedir, inv):
+        self._inventory = inv
+        self.basedir = basedir
+        self.path2id = inv.path2id
+
+    def __repr__(self):
+        return "<%s of %s>" % (self.__class__.__name__,
+                               self.basedir)
+
+    def _rel(self, filename):
+        return os.path.join(self.basedir, filename)
+
+    def has_filename(self, filename):
+        return os.path.exists(self._rel(filename))
+
+    def get_file(self, file_id):
+        return file(self._get_store_filename(file_id), 'rb')
+
+    def _get_store_filename(self, file_id):
+        return self._rel(self.id2path(file_id))
+
+    def get_file_size(self, file_id):
+        return os.stat(self._get_store_filename(file_id))[ST_SIZE]
+
+    def get_file_sha1(self, file_id):
+        f = self.get_file(file_id)
+        return sha_file(f)
+
+
+    def file_class(self, filename):
+        if self.path2id(filename):
+            return 'V'
+        elif self.is_ignored(filename):
+            return 'I'
+        else:
+            return '?'
+
+
+    def file_kind(self, filename):
+        if isfile(self._rel(filename)):
+            return 'file'
+        elif isdir(self._rel(filename)):
+            return 'directory'
+        else:
+            return 'unknown'
+
+
+    def list_files(self):
+        """Recursively list all files as (path, class, kind, id).
+
+        Lists, but does not descend into unversioned directories.
+
+        This does not include files that have been deleted in this
+        tree.
+
+        Skips the control directory.
+        """
+        inv = self.inventory
+
+        def descend(from_dir, from_dir_id, dp):
+            ls = os.listdir(dp)
+            ls.sort()
+            for f in ls:
+                if bzrlib.BZRDIR == f:
+                    continue
+
+                # path within tree
+                fp = appendpath(from_dir, f)
+
+                # absolute path
+                fap = appendpath(dp, f)
+                
+                f_ie = inv.get_child(from_dir_id, f)
+                if f_ie:
+                    c = 'V'
+                elif self.is_ignored(fp):
+                    c = 'I'
+                else:
+                    c = '?'
+
+                fk = file_kind(fap)
+
+                if f_ie:
+                    if f_ie.kind != fk:
+                        bailout("file %r entered as kind %r id %r, now of kind %r"
+                                % (fap, f_ie.kind, f_ie.file_id, fk))
+
+                yield fp, c, fk, (f_ie and f_ie.file_id)
+
+                if fk != 'directory':
+                    continue
+
+                if c != 'V':
+                    # don't descend unversioned directories
+                    continue
+                
+                for ff in descend(fp, f_ie.file_id, fap):
+                    yield ff
+
+        for f in descend('', None, self.basedir):
+            yield f
+            
+
+
+    def unknowns(self, path='', dir_id=None):
+        """Yield names of unknown files in this WorkingTree.
+
+        If there are any unknown directories then only the directory is
+        returned, not all its children.  But if there are unknown files
+        under a versioned subdirectory, they are returned.
+
+        Currently returned depth-first, sorted by name within directories.
+        """
+        for fpath, fclass, fkind, fid in self.list_files():
+            if fclass == '?':
+                yield fpath
+                
+
+    def ignored_files(self):
+        for fpath, fclass, fkind, fid in self.list_files():
+            if fclass == 'I':
+                yield fpath
+
+
+    def is_ignored(self, filename):
+        """Check whether the filename matches an ignore pattern."""
+        ## TODO: Take them from a file, not hardcoded
+        ## TODO: Use extended zsh-style globs maybe?
+        ## TODO: Use '**' to match directories?
+        ## TODO: Patterns without / should match in subdirectories?
+        for i in bzrlib.DEFAULT_IGNORE:
+            if fnmatch.fnmatchcase(filename, i):
+                return True
+        return False
+        
+
+        
+        
+
+class RevisionTree(Tree):
+    """Tree viewing a previous revision.
+
+    File text can be retrieved from the text store.
+
+    :todo: Some kind of `__repr__` method, but a good one
+           probably means knowing the branch and revision number,
+           or at least passing a description to the constructor.
+    """
+    
+    def __init__(self, store, inv):
+        self._store = store
+        self._inventory = inv
+
+    def get_file(self, file_id):
+        ie = self._inventory[file_id]
+        f = self._store[ie.text_id]
+        mutter("  get fileid{%s} from %r" % (file_id, self))
+        fs = filesize(f)
+        if ie.text_size is None:
+            note("warning: no text size recorded on %r" % ie)
+        self._check_retrieved(ie, f)
+        return f
+
+    def get_file_size(self, file_id):
+        return self._inventory[file_id].text_size
+
+    def get_file_sha1(self, file_id):
+        ie = self._inventory[file_id]
+        return ie.text_sha1
+
+    def has_filename(self, filename):
+        return bool(self.inventory.path2id(filename))
+
+    def list_files(self):
+        # The only files returned by this are those from the version
+        for path, entry in self.inventory.iter_entries():
+            yield path, 'V', entry.kind, entry.file_id
+
+
+class EmptyTree(Tree):
+    def __init__(self):
+        self._inventory = Inventory()
+
+    def has_filename(self, filename):
+        return False
+
+    def list_files(self):
+        if False:  # just to make it a generator
+            yield None
+    
+
+
+######################################################################
+# diff
+
+# TODO: Merge these two functions into a single one that can operate
+# on either a whole tree or a set of files.
+
+# TODO: Return the diff in order by filename, not by category or in
+# random order.  Can probably be done by lock-stepping through the
+# filenames from both trees.
+
+
+def file_status(filename, old_tree, new_tree):
+    """Return single-letter status, old and new names for a file.
+
+    The complexity here is in deciding how to represent renames;
+    many complex cases are possible.
+    """
+    old_inv = old_tree.inventory
+    new_inv = new_tree.inventory
+    new_id = new_inv.path2id(filename)
+    old_id = old_inv.path2id(filename)
+
+    if not new_id and not old_id:
+        # easy: doesn't exist in either; not versioned at all
+        if new_tree.is_ignored(filename):
+            return 'I', None, None
+        else:
+            return '?', None, None
+    elif new_id:
+        # There is now a file of this name, great.
+        pass
+    else:
+        # There is no longer a file of this name, but we can describe
+        # what happened to the file that used to have
+        # this name.  There are two possibilities: either it was
+        # deleted entirely, or renamed.
+        assert old_id
+        if new_inv.has_id(old_id):
+            return 'X', old_inv.id2path(old_id), new_inv.id2path(old_id)
+        else:
+            return 'D', old_inv.id2path(old_id), None
+
+    # if the file_id is new in this revision, it is added
+    if new_id and not old_inv.has_id(new_id):
+        return 'A'
+
+    # if there used to be a file of this name, but that ID has now
+    # disappeared, it is deleted
+    if old_id and not new_inv.has_id(old_id):
+        return 'D'
+
+    return 'wtf?'
+
+    
+

=== added file 'bzrlib/xml.py'
--- bzrlib/xml.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml.py	2005-03-09 04:08:15 +0000
@@ -0,0 +1,46 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""XML externalization support."""
+
+
+__copyright__ = "Copyright (C) 2005 Canonical Ltd."
+__author__ = "Martin Pool <mbp@canonical.com>"
+
+try:
+    from cElementTree import Element, ElementTree, SubElement
+except ImportError:
+    from ElementTree import Element, ElementTree, SubElement
+
+import os, time
+from trace import mutter
+
+class XMLMixin:
+    def to_element(self):
+        raise Exception("XMLMixin.to_element must be overridden in concrete classes")
+    
+    def write_xml(self, f):
+        ElementTree(self.to_element()).write(f, 'utf-8')
+        f.write('\n')
+
+    def read_xml(cls, f):
+        return cls.from_element(ElementTree().parse(f))
+
+    read_xml = classmethod(read_xml)
+
+
+

